package network.functions;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import edu.uci.ics.jung.algorithms.shortestpath.DistanceStatistics;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;

import network.graph.MyGraph;

public class GlobalFunctions {
	
	
	public static int getMaxFullyDisconnectedDegree(MyGraph g) {
		return (int) IndividualFunctions.getFullyDisconnectedDegrees(g).getMax();
	}
	
	public static HashSet<Integer> getReachableNodes(MyGraph g, int node) {
		
		HashSet<Integer> visited = new HashSet<Integer>();
		LinkedList<Integer> list = new LinkedList<Integer>();
		list.add(node);
		
		while(!list.isEmpty()) {
			int v = list.removeFirst();
			visited.add(v);
			Integer neighs[] = g.getNeighs(v);
			for(int i=0;i<neighs.length;i++)
				if(!visited.contains(neighs[i])){
					list.addLast(neighs[i]);
				}
		}
		
		return visited;
		
	}
	
	public static void updateNodesComponentLabel(MyGraph g, HashSet<Integer> nodes, int cLabel) {
		Iterator<Integer> iNodes = nodes.iterator();
		while(iNodes.hasNext())
			g.getNode(iNodes.next()).setComponentLabel(cLabel);
	}
	
	public static int getNumberOfComponents(MyGraph g) {
		
		if(g.COMPUTE_COMPONENTS_DYNAMICLY)
			return g.getComponentsSize().size();
		
		int n=0;
		
		HashSet<Integer> nodes = g.getNodeSet();
		
		while(!nodes.isEmpty()) {
			int v = nodes.iterator().next();
			int compLabel = g.getNode(v).getComponentLabel();
			HashSet<Integer> reachedNodes = GlobalFunctions.getReachableNodes(g, v);
			n++;
			System.out.println("component " + n + ":" + reachedNodes.size());
			
			//update the component label of the reached nodes
			GlobalFunctions.updateNodesComponentLabel(g, reachedNodes, compLabel);
			
			nodes.removeAll(reachedNodes);
		}
		
		return n;
	}
	
	public static void updateComponentsEdgeAdded(MyGraph g, int v1, int v2) {
		//if labels are identical, do nothing
		int c1 = g.getNode(v1).getComponentLabel();
		int c2 = g.getNode(v2).getComponentLabel();
		if(c1 != c2) {
			if(g.getComponentSize(c1) < g.getComponentSize(c2)) {
				HashSet<Integer> reachedNodes = GlobalFunctions.getReachableNodes(g, v1);
				GlobalFunctions.updateNodesComponentLabel(g, reachedNodes, c2);		
				g.mergeComponents(c2, c1);
			}
			else {
				HashSet<Integer> reachedNodes = GlobalFunctions.getReachableNodes(g, v2);
				GlobalFunctions.updateNodesComponentLabel(g, reachedNodes, c1);
				g.mergeComponents(c1, c2);
			}

		}
	}
	
	public static void updateComponentsEdgeRemoved(MyGraph g, int v1, int v2) {	
		HashSet<Integer> reachedNodes = GlobalFunctions.getReachableNodes(g, v1);
		g.updateComponentSize(g.getNode(v1).getComponentLabel(), reachedNodes.size());
		if(!reachedNodes.contains(v2)) {
			int c2 = g.getNewComponentLabel();
			reachedNodes = GlobalFunctions.getReachableNodes(g, v2);
			GlobalFunctions.updateNodesComponentLabel(g, reachedNodes, c2);
			g.updateComponentSize(c2, reachedNodes.size());
		}
	}
	
	public static double getAssortativityDegree(MyGraph g) {
		double p1 = 0;
		double p2 = 0;
		double p3 = 0;
		double ne = g.getNumEdges();
		double r = 0;
		
		Integer idNodes[] = g.getNodes();
		int n = idNodes.length;
		
		for(int i=0; i<n; i++) {
			for(int j=i+1; j<n; j++) {
				if(g.existsEdge(idNodes[i], idNodes[j])) {
					p1 += (1/ne)*g.getDegree(idNodes[i])*g.getDegree(idNodes[j]);
					p2 += (1/ne)*0.5*(g.getDegree(idNodes[i])+g.getDegree(idNodes[j]));
					p3 += (1/ne)*0.5*(Math.pow(g.getDegree(idNodes[i]),2)+Math.pow(g.getDegree(idNodes[j]),2));
				}
			}
		}
		r = (p1 - Math.pow(p2,2))/(p3 - Math.pow(p2,2));
		return r;
	}
	
	public static double getCC(MyGraph g) {
		DescriptiveStatistics ds = new DescriptiveStatistics();
		
		Integer nodes[] = g.getNodes();
		for(int i=0; i<nodes.length; i++) {
			double cc = IndividualFunctions.clusteringCoefficient(g, nodes[i]);
			if(cc >= 0)
				ds.addValue(cc);
		}
		
		return ds.getMean();
		
	}
	
	public static double getDiameter(MyGraph g) {
		return DistanceStatistics.diameter(g.getGraphJung(), new UnweightedShortestPath(g.getGraphJung()), true);
	}
	
	

	
	public static void main(String[] args) {
		MyGraph g = new MyGraph("Teste");
		
		g.addEdge(1, 2);
		g.addEdge(1, 4);
		g.addEdge(4, 5);
		
		g.addEdge(3, 6);
		g.addEdge(6, 9);
		
		g.addEdge(6, 7);
		g.addEdge(7, 11);

		
		System.out.println(GlobalFunctions.getNumberOfComponents(g));

	}

}
